//*******************************************************************************************************
//
// This defines every object generated by POVray in this apng:
// 
// the board, and flat pegs and links of both colors.
// 
// 


// #declare Table =
// box {<TABLE_LEFT, TABLE_BOTTOM, TABLE_BACK>, <TABLE_RIGHT, TABLE_SURFACE, TABLE_FRONT> 
//                                              
//      texture {T_Wood7 rotate <0, 90, 0>}}    


#declare Peg =
  union
  {
    difference
    { 
      cylinder { 0, y*PegHeight, PegOuter }
      cylinder { y*(PegHeight+epsilon), y*(PegHeight/2), PegInner}
    }
    cylinder { y*epsilon, -y*HoleDepth, HoleRad }
    sphere { -y*HoleDepth, HoleRad}
  }
  
#declare WhitePeg =
  object
  { Peg
    pigment { rgb <1, 1, 1> }
    finish { F_MetalA }
  }

#declare BlackPeg =
  object
  { Peg
    pigment { rgb <0.00, 0.00, 0.00> }
    finish { Phong_Dull }
  }

#declare LinkRiser =
  difference
{
  cylinder
  {
    <-LinkRiserThickness/2, LinkRiserCenter, 0>,
    <LinkRiserThickness/2, LinkRiserCenter, 0>, LinkRiserRadius
  }
  
  box  // Cut away all of the riser cylinder except the sliver along the top.
  {
    <-epsilon-(LinkRiserThickness/2), PegHeight+LinkRad, LinkRiserRadius+epsilon>,
    <epsilon+(LinkRiserThickness/2), (LinkRiserCenter-LinkRiserRadius)-epsilon, -LinkRiserRadius-epsilon>
  }
}

/*  
#declare Link =
  union
  {
    object { LinkRiser }
    box { <-LinkRad, PegHeight+LinkRad, HLL>, <LinkRad, PegHeight, -HLL> }
    box { <-LinkRad, PegHeight+epsilon, HLU>, <LinkRad, PegHeight-LinkRad, -HLU> }
    box { <-LinkRad, PegHeight+epsilon, HLL>, <LinkRad, PegHeight-LinkRad, HLL-LinkRad> }
    box { <-LinkRad, PegHeight+epsilon, -HLL+LinkRad>, <LinkRad, PegHeight-LinkRad, -HLL> }
  }
*/


#declare TrimBox1 =          // to trim off the four corners of a link
  box { <0, PegHeight+LinkRad+epsilon, 0>, <TrimB+epsilon, PegHeight-LinkRad-epsilon, -TrimA-epsilon> }
    
#declare TrimBox2 = 
  box { <-TrimB-epsilon, PegHeight+LinkRad+epsilon, 0>, <0, PegHeight-LinkRad-epsilon, -TrimA-epsilon> }
    
#declare TrimBoxes =
  union
  {
    object { TrimBox1 rotate y*(-TrimAlpha) translate <0, 0. (HLL-LinkRad)+TrimC> }
    object { TrimBox1 rotate y*(180-TrimAlpha) translate <0, 0, (-HLL+LinkRad)-TrimC> }
    object { TrimBox2 rotate y*TrimAlpha translate <0, 0, (HLL-LinkRad)+TrimC> }
    object { TrimBox2 rotate y*(180+TrimAlpha) translate <0, 0, (-HLL+LinkRad)-TrimC> }
  }


#declare Link =
  difference
  {
    union
    {
      object { LinkRiser }
      box { <-LinkRad, PegHeight+LinkRad, HLL>, <LinkRad, PegHeight, -HLL> }
      box { <-LinkRad, PegHeight+epsilon, HLU>, <LinkRad, PegHeight-LinkRad, -HLU> }
      box { <-LinkRad, PegHeight+epsilon, HLL>, <LinkRad, PegHeight-LinkRad, HLL-LinkRad> }
      box { <-LinkRad, PegHeight+epsilon, -HLL+LinkRad>, <LinkRad, PegHeight-LinkRad, -HLL> }
    }
    object { TrimBoxes }

}

#declare BlackLink =
  object
  { Link
    pigment { rgb <0.00, 0.00, 0.00> }
    finish { Phong_Dull }
  }

#declare WhiteLink =
  object
  { Link
    pigment { rgb <1, 1, 1> }
    finish { F_MetalA }
  }     
      

//Coordlet and Coordnum vary from 1 to 24 (or 1...23 or 2...23 for link positions)

#macro PegPos(Coordlet, Coordnum)
  <LetterStart+separation*(Coordlet-1), 0, NumberStart-separation*(Coordnum-1)>
#end

#macro Link12Pos(Coordlet, Coordnum)
  <LetterStart+separation*(Coordlet-0.5), 0, NumberStart-separation*(Coordnum-1)>
#end

#macro Link34Pos(Coordlet, Coordnum)
  <LetterStart+separation*(Coordlet-1), 0, NumberStart-separation*(Coordnum-0.5)>
#end


#declare TwixtBoard =
  difference
  {
    union
    { prism
      {
        linear_spline
        linear_sweep
        0, -3*HoleDepth, 9,
        <-11*separation, 12*separation>, <11*separation, 12*separation>,
        <12*separation, 11*separation>, <12*separation, -11*separation>,
        <11*separation, -12*separation>, <-11*separation, -12*separation>,
        <-12*separation, -11*separation>, <-12*separation, 11*separation>,
        <-11*separation, 12*separation>
        texture { T_Wood11 }
      }
      cylinder                              // rounded corners for the board
      {
        <-11*separation, -3*HoleDepth, -11*separation>, <-11*separation, 0, -11*separation>, separation
        texture { T_Wood11 }
      }
      cylinder
      {
        <11*separation, -3*HoleDepth, -11*separation>, <11*separation, 0, -11*separation>, separation
        texture { T_Wood11 }
      }
      cylinder
      {
        <11*separation, -3*HoleDepth, 11*separation>, <11*separation, 0, 11*separation>, separation
        texture { T_Wood11 }
      }
      cylinder
      {
        <-11*separation, -3*HoleDepth, 11*separation>, <-11*separation, 0, 11*separation>, separation
        texture { T_Wood11 }
      }
      box                                   // border lines
      {
        <-11*separation-(BorderWidth/2), 0, -10.5*separation-PegOuter>,
        <-11*separation+(BorderWidth/2), epsilon, 10.5*separation+PegOuter>
        pigment { rgb <0.05, 0.05, 0.05> } finish { specular .2 }
      }
      box
      {
        <11*separation-(BorderWidth/2), 0, -10.5*separation-PegOuter>,
        <11*separation+(BorderWidth/2), epsilon, 10.5*separation+PegOuter>
        pigment { rgb <0.05, 0.05, 0.05> } finish { specular .2 }
      }
      box
      {
        <-10.5*separation-PegOuter, 0, -11*separation-(BorderWidth/2)>,
        <10.5*separation+PegOuter, epsilon, -11*separation+(BorderWidth/2)>
        pigment { rgb <1, 1, 1> } finish { specular .2 }
      }
      box
      {
        <-10.5*separation-PegOuter, 0, 11*separation-(BorderWidth/2)>,
        <10.5*separation+PegOuter, epsilon, 11*separation+(BorderWidth/2)>
        pigment { rgb <1, 1, 1> } finish { specular .2 }
      }
      box                                                // diagonal guide lines
      { 
        <-LineWidth/2, 0, 0>,
//        <LineWidth/2, epsilon, 10*s5*separation> for full length guide lines
        <LineWidth/2, epsilon, 7*s5*separation>           // for "two diamonds" 
        rotate y*Theta
        translate <-10.5*separation, 0, -10.5*separation>
        pigment { SlateBlue } finish { specular .3 }
      }
      box                                              
      { 
        <-LineWidth/2, 0, 0>,
//        <LineWidth/2, epsilon, 10*s5*separation> for full length guide lines
        <LineWidth/2, epsilon, 7*s5*separation>           // for "two diamonds" 
        rotate y*(90-Theta)
        translate <-10.5*separation, 0, -10.5*separation>
        pigment { SlateBlue } finish { specular .3 }
      }
      box                                                
      { 
        <-LineWidth/2, 0, 0>,
//        <LineWidth/2, epsilon, 10*s5*separation>
        <LineWidth/2, epsilon, 7*s5*separation>           // for "two diamonds" 
        rotate y*(0-Theta)
        translate <10.5*separation, 0, -10.5*separation>
        pigment { SlateBlue } finish { specular .3 }
      }
      box                                                
      { 
        <-LineWidth/2, 0, 0>,
//        <LineWidth/2, epsilon, 10*s5*separation>
        <LineWidth/2, epsilon, 7*s5*separation>           // for "two diamonds" 
        rotate y*(Theta-90)
        translate <10.5*separation, 0, -10.5*separation>
        pigment { SlateBlue } finish { specular .3 }
      }
      box                                                
      { 
        <-LineWidth/2, 0, 0>,
//        <LineWidth/2, epsilon, 10*s5*separation>
        <LineWidth/2, epsilon, 7*s5*separation>           // for "two diamonds" 
        rotate y*(90+Theta)
        translate <-10.5*separation, 0, 10.5*separation>
        pigment { SlateBlue } finish { specular .3 }
      }
      box
      { 
        <-LineWidth/2, 0, 0>,
//        <LineWidth/2, epsilon, 10*s5*separation>
        <LineWidth/2, epsilon, 7*s5*separation>           // for "two diamonds" 
        rotate y*(180-Theta)
        translate <-10.5*separation, 0, 10.5*separation>
        pigment { SlateBlue } finish { specular .3 }
      }
      box
      { 
        <-LineWidth/2, 0, 0>,
//        <LineWidth/2, epsilon, 10*s5*separation>
        <LineWidth/2, epsilon, 7*s5*separation>           // for "two diamonds" 
        rotate y*(-90-Theta)
        translate <10.5*separation, 0, 10.5*separation>
        pigment { SlateBlue } finish { specular .3 }
      }
      box
      { 
        <-LineWidth/2, 0, 0>,
//        <LineWidth/2, epsilon, 10*s5*separation>
        <LineWidth/2, epsilon, 7*s5*separation>           // for "two diamonds" 
        rotate y*(Theta-180)
        translate <10.5*separation, 0, 10.5*separation>
        pigment { SlateBlue } finish { specular .3 }
      }
//      #declare Ind = 1;                              // Grid labels
//      #while (Ind < 25)
//        #declare Letter =  substr(Letters, Ind, 1);
//        #declare Number =  substr(Numbers, 2*Ind-1, 2);
//        text { ttf "crystal.ttf" Letter epsilon, 0
//               scale x*0.8
//               scale z*2
//               translate <(Ind-12.6)*separation, 0, 12*separation>
//             }
//        text { ttf "crystal.ttf" Letter epsilon, 0
//               scale x*0.8
//               scale z*2
//               translate <((Ind-12.6)*separation), 0, -12*separation>
//             }
//        text { ttf "crystal.ttf" Number epsilon, 0
//              scale x*0.8
//               scale z*2
//               translate <-12.75*separation, 0, (12.4-Ind)*separation>
//             }             
//        text { ttf "crystal.ttf" Number epsilon, 0
//               scale x*0.8
//               scale z*2
//               translate <11.75*separation, 0, (12.4-Ind)*separation>
//             }
//        #declare Ind = Ind+1;
//      #end             
    }
    #declare IndX = 1;                               // peg holes
    #while (IndX < 25)
      #declare IndZ = 1;
      #while (IndZ < 25)
        #if (!((IndX = 1 & IndZ = 1)|(IndX = 1 & IndZ = 24)|(IndX = 24 & IndZ = 1)|(IndX = 24 & IndZ = 24)))
          cylinder { PegPos(IndX, IndZ)-y*(HoleDepth+HoleRad), PegPos(IndX, IndZ)+y*1, HoleRad }
        #end
        #declare IndZ = IndZ+1;
      #end
      #declare IndX = IndX+1;
    #end
  }





















