// ********************************************************************************************************
//
// This file is necessarily distinct from ap_constants.inc because some of this animation is static.
// The first thing is to determine whether we need to bother loading all those other include files.
// This file finds out if we are going to generate a new image or not.
//
// If not, the ap.pov code will generate a black frame, which will be replaced with an image
// by the post-frame command.
//
// Every distinct scene (or interval) in this video, whether static or animated,
// is uniquely numbered starting at 0.
// 
// This file defines two arrays with the following information about each interval:
//   type of interval: animated, static POVray, or static Inkscape
//   start time for this interval (always a multiple of 0.1 seconds)
//   
// If a cp command is necessary, the string which is the cp command is generated here,
// but the fix.sh file is written to within the ap.pov file, not here.




#declare FPS = 60;                // frames per second
#declare TimeVariation = 0.001;   // allow for possible numerical discrepancy

#declare NumScenes = 3;               // before, during, after
#declare StartTimes = array [NumScenes]
{
    0, 1, 9     // 10 seconds total. First second is static before,
}               // last second is static after

// A value of 0 means this scene is animated, 1 means this is static POV, 2 means Inkscape.
// There are no Inkscape imasges for this animated PNG file.

#declare SceneType = array [NumScenes]
{
  1,     0,     1    
}

#declare FinalTime = 10;;
#declare ThisTime = clock*FinalTime;

#declare ThisFrameIndex= int((ThisTime + TimeVariation)*FPS) ;




#declare IntervalIndex =0;                  // 1 less than the corresponding interval number
#declare IntervalFound = false;


#while (!IntervalFound)        // Done this way because I don't know that shortcut conditionals work

  #if  ( ThisTime > (StartTimes[IntervalIndex+1] - TimeVariation))
  
    #declare IntervalIndex = IntervalIndex+1;     // This loop finds the current Frame Index
    #if (IntervalIndex = NumScenes-1)          // without exceeding the bounds of the array.
      #declare IntervalFound = true;
    #end
  
  #else #declare IntervalFound = true;
  #end  
  
#end

// In order to create the desired cp command, we need these string values:
//
// FrameFullName  
//           is the name of the frame that this clock iteration is responsible for creating.
//           It consists of three parts.
//           The first part should be what povray.ini defines as the output file path name.
//           The second part is the frame number, padded on the left with zeroes if necessary.
//           The third part is the frame suffix which is .png
//
// RptName   is the name of a file which will be repeatedly copied into frames for static intervals.
//           This is either an Inkscape generated file, in which case it is always the source file,
//           Or it is the first frame of a POVray generated static interval.
//           The first frame of such an interval will use RptName as the destination of the cp.
//           On subsequent frames of that interval, RptName will indicate the source file.
//           This string also consists of three parts.
//


#declare FramePrefix ="./images/animated/apng_long_move"; 
#declare FrameSuffix = ".png";
#declare PaddedFrameNumber = str(ThisFrameIndex+1, -3, 0);               // 1 more than index
#declare FrameFullName = concat(FramePrefix, PaddedFrameNumber, FrameSuffix);
    
#declare StaticPrefix = "./images/static/static";
#declare PaddedStaticIndex = str(IntervalIndex, -1, 0);  // No need to add 1 here. First file is 0
#declare RptName = concat(StaticPrefix, PaddedStaticIndex, FrameSuffix);

// Is this the first frame of this interval?

#declare IsFirstFrame = (ThisTime < (StartTimes[IntervalIndex] + TimeVariation));
    
#declare ThisType = SceneType[IntervalIndex];   // which of the 3 types is this interval

#switch (ThisType)
  #case (0)
    #warning "In ap_skip.inc ANIMATING\n"
    #declare Animating = true;
    #declare DoThis    = true;      // DoThis means this animation iteration will generate a frame.
  #break
  #case (1)
    #declare Animating = false;
    #declare StaticPOV = true;
    #if (IsFirstFrame)
      #warning concat( "In ap_skip.inc First frame, copy ", FrameFullName, " to ", RptName, "\n")
      #declare DoThis  = true;
      #declare SourceFile = FrameFullName;
      #declare DestFile = RptName;
      #declare CpCommand = concat("cp -f -v ", SourceFile," ", DestFile);
    #else
      #warning concat( "In ap_skip.inc black frame, copy ", RptName, " to ", FrameFullName, "\n")
      #declare DoThis = false;
      #declare SourceFile = RptName;
      #declare DestFile = FrameFullName;
      #declare CpCommand = concat("cp -f -v ", SourceFile," ", DestFile);
    #end
  #break
  #case (2)                         // this clause should not happen
    #error "THIS SHOULD NOT HAPPEN ***** No Inkscape files ih this animateion. *****\n"
    #declare Animating = false;
    #declare DoThis = false;
    #declare StaticPOV = false;
    #declare SourceFile = RptName;
    #declare DestFile = FrameFullName;
//    #declare Inkscape = true;      // probably unnecessary, since this is never declared false
  #break
  #else
    #error "Bad value for ThisType"
#end
    

  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  